dom_extractor = """

function getDOMInfoOfElement(element) {
  
  var rect = element.getBoundingClientRect();
  if (rect.width == 0 || rect.height == 0) return;
  var answer = {
    tag: element.tagName,
    left: rect.left, top: rect.top,
    width: rect.width, height: rect.height,
    children: [],
    id: element.id,
  };
  if (element instanceof HTMLInputElement) {
    var inputType = element.type;
    answer.tag += '_' + inputType;
    if (inputType === 'checkbox' || inputType === 'radio') {
      answer.value = element.checked;
    } else {
      answer.value = element.value;
    }
  } else if (element instanceof HTMLTextAreaElement) {
    answer.value = element.value;
  }
  var filteredChildNodes = [], textOnly = true;
  element.childNodes.forEach(function (child) {
    if (child instanceof Text) {
      if (!/^\s*$/.test(child.data)) {
        filteredChildNodes.push(child);
      }
    } else if (child instanceof Element) {
      filteredChildNodes.push(child);
      textOnly = false;
    }
  });
  if (textOnly) {
    answer.text = filteredChildNodes.map(function (x) {
      return x.data.trim();
    }).join(' ');
  } else {
    filteredChildNodes.forEach(function (child) {
      if (child instanceof Text) {
        addDOMInfosOfTextNode(child, answer.children);
      } else {
        child = getDOMInfoOfElement(child);
        if (child !== undefined)
          answer.children.push(child);
      }
    });
  }
  return answer;
}

return getDOMInfoOfElement(document.body)












function addDOMInfosOfTextNode(textNode, collection) {
  // Break the text node into multiple nodes
  // Each node only occupies a single rectangle boundary
  var range = document.createRange();
  range.selectNodeContents(textNode);
  var absolute_start = range.startOffset, absolute_end = range.endOffset;
  var start = absolute_start;
  var itr = 0;
  while (start < absolute_end) {
    // Binary search on the next end point
    var end_lower_bound = start + 1,
        end_upper_bound = absolute_end,
        l = range.getClientRects().length,
        end = Math.floor((end_lower_bound * (l-1) + end_upper_bound) / l);
    while (end_lower_bound <= end_upper_bound) {
      range.setEnd(textNode, end);
      if (range.getClientRects().length == 1) {
        end_lower_bound = end + 1;
        end = Math.min(end_lower_bound + 5, Math.floor((end_lower_bound + end_upper_bound) / 2));
      } else {
        end_upper_bound = end - 1;
        end = Math.max(end_upper_bound - 5, Math.floor((end_lower_bound + end_upper_bound) / 2));
      }
      if (itr++ > 1000) throwTextNodeError('Text node computation stuck in an infinite loop');
    }
    range.setEnd(textNode, end);
    var rects = range.getClientRects();
    if (rects.length !== 1) throwTextNodeError('Text node computation incorrect');
    var rect = rects[0], text = textNode.data.substring(start, end).trim();
    if (rect.width > 0 && rect.height > 0 && text) {
      var answer = {
        tag: "t",
        left: rect.left, top: rect.top,
        width: rect.width, height: rect.height,
        children: [],
        text: text,
      };
      collection.push(answer);
    }
    start = end;
    range.setEnd(textNode, absolute_end);
    range.setStart(textNode, start);
    if (itr++ > 1000) throwTextNodeError('Text node computation stuck in an infinite loop');
  }
}


"""